<?
// PostgreSQL db 사용을 위한 두개의 클래스
// 하나는 접속관련 클래스고, 
// 다른 하나는 쿼리 결과 조작에 관련한 클래스임

/*
저작권: 이 파일에서 다루고 있는 모든 코드는 다음 조건 아래서, 
        제작자의 동의 없이, 상업적이든 그렇지않든 마음대로 사용할 수 있습니다.
조건1: 코드가 수정되지 않고 부분적으로 사용될 경우는 반드시
       이 저작권을 명시해야합니다.
조건2: 코드가 수정되었을 경우는 변경된 부분을 반드시 명시하고,
       그 변경된 부분에 대한 저작권은 변경자의 제량에 맡깁니다.
조건3: 이 코드를 사용함으로 인해 발생하는 불이익에 대한 책임은 제작자에게 없습니다.

제1차 제작자: 김상기 <ioseph@uri.sarang.net>
*/


// 접속 클래스 
/*
맴버함수들:
	pgsql(): 생성자, db 연결
	exec($query_string): 쿼리실행, 리턴값은 pg_query()의 리턴값
	close(): db 연결 끊기
	pgarr2arr($pgarray_string) : 배열자료형태로 나타난 문자열을 php 배열로 만듬
맴버변수들:
	conn : 접속핸들
	connection_string: 접속문자열
	client_encoding: 클라이언트 문자셋

사용개요: 
	$db = new pgsql();
	$res = $db->exec("select * from pg_class");
	$result = new pg_result($db->conn, $res);
	echo $result->printtable();
	$db->close();
*/
class pgsql {
	var $conn;
	var $connection_string = "host=localhost";
	var $client_encoding = "EUC_KR";
	function pgsql(){
		global $isdebug, $ispopupwin;
		$this->conn = @pg_connect($this->connection_string);
		if(!$this->conn){
			$errormsg = "<p>데이터 베이스 서버 접속 실패.</p>";
			$errormsg .= "<p>$php_errormsg</p>";
			include("html/error.php");
			exit;
		}
		pg_exec($this->conn, "set CLIENT_ENCODING = '$this->client_encoding'");
	}
	function exec($str){
		global $isdebug, $debug_msg, $ispopupwin, $inc;
		$res = @pg_query($this->conn, $str);
		if(!$res){
			$errormsg = "<p>데이터베이스 질의 오류</p>";
			if($isdebug) {
				$errormsg .= "<p>\nQuery:\n$str\n</p>";
			}
			$errormsg .= "<p>" . pg_errormessage($this->conn) . "</p>\n";
			include("html/error.php");
			exit;
		}
		if($isdebug){
		    // debug 작업을 위해, debug_msg 전역문자열변수에다가 쿼리들 다 담아둠
			$debug_msg .= "Query:\n$str\n";
			if(!$res) $debug_msg .= pg_errormessage($this->conn) . "\n";
			$debug_msg .= "\n";
		}
		return $res;
	}
	function close(){
		pg_close($this->conn);
	}

	function pgarr2arr($str){
		$arr = explode(",", ereg_replace("[\{\}\"]", "", $str));
		return $arr;
	}
}

// 쿼리 결과 조작 클래스
/*
맴버함수들:
	pg_result($dbhandle, $query_result): 생성자
	close() : 쿼리 결과 포인트 해제
	getmaxrow() : 
	getmaxcol() : 컬럼,로우 갯수 구함기
	getheader() : 쿼리 결과의 메타정보(컬럼이름, 크기, 자료형) 구하기
	printrow() : 하나의 row를 html <table> tag의 <td>...</td>... 문자열로 만듬.
	printtr() : printrow() 결과 앞뒤에 <tr></tr> 문자열 추가
	printheadtr() : <table>의 thead 부분 문자열 만들기
	printtable() : printheadtr(), printtr() 두 함수를 함쳐 전체 <table>..</table> 문자열 만들기
	fetchrow() : 하나의 row를 단순 배열로 만듬
	mkselect() : html <select> tag의 <option> 부분을 만들기 위한 함수
	mkarray() : 쿼리 결과 전체를 배열로 만듬.
맴버 변수들:
	conn : db 연결 핸들
	res : query result
	maxrow : 총 row 수
	maxcol : 총 컬럼수
사용법:
	html 모양과 관계되는 부분이기에, table 구조 단위로 함수를 분리했음, 
	디자인이 바뀌어야할 필요가 있는 부분만 상속-재정의해서 사용하면 됨.
	* 제작자 넋두리: 급조된 놈인지라, 범용적이지 못함.
*/
class pg_result {
	var $conn, $res, $maxrow, $maxcol;

	function pg_result($conn, $result){
		$this->conn = $conn;
		$this->res = $result;
		$this->maxrow = pg_numrows($this->res);
		$this->maxcol = pg_numfields($this->res);
	}
	function close(){
		pg_freeresult($this->res);
	}
	function getmaxrow(){
		return pg_numrows($this->res);
	}
	function getmaxcol(){
		return pg_numfields($this->res);
	}

	function getheader(){
		$maxcol = pg_numfields($this->res);
		for($i=0;$i<$maxcol;$i++){
			$arr["name"] = pg_FieldName($this->res, $i);
			$arr["size"] = pg_FieldSize($this->res, $i);
			$arr["type"] = pg_FieldType($this->res, $i);
			$rearr[$i] = $arr;
		}
		return $rearr;
	}

	function printrow($row){
		$str = "";
		for($j=0;$j<$this->maxcol;$j++){
			if($row[$j] == "") $row[$j] = "&nbsp;";
			$str .= "<td";
			$str .= ">$row[$j]</td>";
		}
		return $str;
	}

	function printtr($tinfo){
		$str = "";
		for($i=0;$i<$this->maxrow;$i++){
			$row = pg_fetch_row($this->res, $i);
			$str .= "<tr class='$tinfo->class";
			if($i % 2) $str .= "_eventr";
			else $str .= "_oddtr";
			$str .= "'>";
			$str .= $this->printrow($row);
			$str .= "</tr>\n";
		}
		return $str;
	}

	function printheadtr($tinfo){
        $str .= "<tr class='";
		$str .= $tinfo->class;
		$str .= "_headtr'>";
		for($i=0;$i<$this->maxcol;$i++){
			$str .= "<th>";
			$str .= pg_FieldName($this->res, $i);
			$str .= "</th>";
		}
		$str .= "</tr>\n";
		for($i=0;$i<$this->maxcol;$i++){
			$str .= "<col class='";
			$str .= $tinfo->class;
			$str .= "_col$i'>";
		}
		$str .= "\n";
		return $str;
	}

	function printtable($tinfo = ""){
		$maxcol = pg_numfields($this->res);
		$maxrow = pg_numrows($this->res);
		$this->maxcol = $maxcol;
		$this->maxrol = $maxrol;
		$captionstr = "";

		$tinfo->class || $tinfo->class = "tbdefault";
		$tinfo->border || $tinfo->border = "0";
		if($tinfo->cellpadding == "") $tinfo->cellpadding = "3";
		if($tinfo->cellspacing == "") $tinfo->cellspacing = "1";

		if($tinfo){
			while(list($key, $val) = each($tinfo)){
				if($key != "caption") $toption .= " $key='$val'";
				else $captionstr = "<caption>$val</caption>\n";
			}
		}
		$str = "<table$toption>$captionstr\n";


		$str .= $this->printheadtr($tinfo);


		if($maxrow){
			$str .= $this->printtr($tinfo);
		}
		else{
			$str .= "<tr><td colspan=$maxcol class='$tinfo->class" .
				"_nodata'>해당 자료 없음</td></tr>\n";
		}

		$str .= "</table>\n";
		return $str;
	}

	function fetchrow($i = 0){
		return pg_fetch_row($this->res, $i);
	}

	function mkselect($head){
		$str = $head;
		$i = 0;
		while($row = @pg_fetch_row($this->res, $i)){
			$str .= "$row[0]\n";
			$i++;
		}
		$str .= "</select>\n";
		return $str;
	}

	function mkarray(){
		$i=0;
		while($row = @pg_fetch_row($this->res, $i++)){
			$k = $row[0];
			$v = $row[1];
			$arr[$k] = $v;
		}
		return $arr;
	}
}
?>
